package com.maenle.bump.util

import android.content.Context
import android.os.Handler
import android.os.Looper
import android.util.Log
import com.google.firebase.ktx.Firebase
import com.google.firebase.messaging.FirebaseMessaging
import org.json.JSONArray
import org.json.JSONObject
import java.io.File

class BumpProcessor constructor(context: Context) {

    private var rest: RestSingleton = RestSingleton.getInstance(context)
    private lateinit var log: JSONArray


    init {
        log = getLog(context)
    }

    companion object {
        @Volatile
        private var INSTANCE: BumpProcessor? = null
        fun getInstance(context: Context) =
            INSTANCE ?: synchronized(this) {
                INSTANCE ?: BumpProcessor(context).also {
                    INSTANCE = it
                }
            }
    }

    private fun updateLog(context: Context, list: JSONArray) {
        for(i in 0 until list.length()) {
            var exists = false
            for(j in 0 until log.length()) {
                if (list[i] == log[j]) {
                    exists = true
                    continue
                }
            }
            exists.let {
                log.put(list[i])
                addToLog(context, list[i] as JSONObject)
            }
        }
    }

    private fun getSecret(context: Context):String? {
        val local = LocalData(context)
        return local.code
    }

    fun startUpdateHandler(context: Context) {

        val delay: Long = 30_000 //milliseconds
        val handler = Handler(Looper.getMainLooper())

        handler.postDelayed(object : Runnable {
            override fun run() {
                updateFromServer(context)
                handler.postDelayed(this, delay)
            }
        }, delay)
    }

    private fun updateFromServer(context: Context) {
        LocalData(context).code?.let {
            val messenger = MessageProcessor(it)
            rest.list(messenger.sender) { list -> updateLog(context, list) }
        }
    }

    private fun addToLog(context: Context, line: JSONObject) {
        val logFile = File(context.filesDir, ".bump_log")
        logFile.appendText(line.toString())
    }

    fun addSecret(context: Context, newCode: String): Boolean {
        return if(!checkCodeValidity(newCode)) {
            false
        } else {
            val local = LocalData(context)
            local.code = newCode

            setFirebaseToken(context)
            true
        }
    }

    private fun checkCodeValidity(newCode: String): Boolean {
        val validChars = newCode.filter{ it in 'A'..'Z' || it in 'a'..'z' || it == '-'}.length
        return newCode.length == validChars && newCode.length >= 32
    }

    private fun getLog(context: Context): JSONArray {
        val logFile = File(context.filesDir, ".bump_log")
        val log = JSONArray()
        !logFile.exists().let {
            logFile.createNewFile()
        }
        for(line in logFile.readLines()) {
            log.put(JSONObject(line))
        }
        return log
    }

    fun setFirebaseToken(context: Context) {
        val tokenTask = FirebaseMessaging.getInstance().token
        val secret = LocalData(context).code
        val messenger = secret?.let { MessageProcessor(it) }
        tokenTask.addOnSuccessListener {  token -> ( messenger?.let { rest.firebase(messenger.sender, token) { result -> Log.d("result", result.toString())} } ) }
    }

}